@@ -72,6 +72,10 @@ module Network.Wai.Handler.Warp (
   , setLogger
   , setServerPushLogger
   , setGracefulShutdownTimeout
+  , setGracefulCloseTimeout1
+  , setGracefulCloseTimeout2
+  , setMaxTotalHeaderLength
+  , setAltSvc
     -- ** Getters
   , getPort
   , getHost
@@ -79,6 +83,8 @@ module Network.Wai.Handler.Warp (
   , getOnClose
   , getOnException
   , getGracefulShutdownTimeout
+  , getGracefulCloseTimeout1
+  , getGracefulCloseTimeout2
     -- ** Exception handler
   , defaultOnException
   , defaultShouldDisplayException
@@ -93,6 +99,7 @@ module Network.Wai.Handler.Warp (
   , pauseTimeout
   , FileInfo(..)
   , getFileInfo
+  , clientCertificate
   , withApplication
   , withApplicationSettings
   , testWithApplication
@@ -121,6 +128,7 @@ module Network.Wai.Handler.Warp (
 import Control.Exception (SomeException, throwIO)
 import Data.Streaming.Network (HostPreference)
 import qualified Data.Vault.Lazy as Vault
+import Data.X509
 import qualified Network.HTTP.Types as H
 import Network.Socket (SockAddr)
 import Network.Wai (Request, Response, vault)
@@ -187,7 +195,13 @@ setOnOpen x y = y { settingsOnOpen = x }
 setOnClose :: (SockAddr -> IO ()) -> Settings -> Settings
 setOnClose x y = y { settingsOnClose = x }
 
--- | Timeout value in seconds. Default value: 30
+-- | "Slow-loris" timeout lower-bound value in seconds.  Connections where
+-- network progress is made less frequently than this may be closed.  In
+-- practice many connections may be allowed to go without progress for up to
+-- twice this amount of time.  Note that this timeout is not applied to
+-- application code, only network progress.
+--
+-- Default value: 30
 --
 -- Since 2.1.0
 setTimeout :: Int -> Settings -> Settings
@@ -378,7 +392,7 @@ setProxyProtocolRequired y = y { settingsProxyProtocol = ProxyProtocolRequired }
 --
 -- WARNING: This is contrary to the PROXY protocol specification and
 -- using it can indicate a security problem with your
--- architecture if the web server is directly accessable
+-- architecture if the web server is directly accessible
 -- to the public, since it would allow easy IP address
 -- spoofing.  However, it can be useful in some cases,
 -- such as if a load balancer health check uses regular
@@ -430,6 +444,20 @@ setGracefulShutdownTimeout :: Maybe Int
                            -> Settings -> Settings
 setGracefulShutdownTimeout time y = y { settingsGracefulShutdownTimeout = time }
 
+-- | Set the maximum header size that Warp will tolerate when using HTTP/1.x.
+--
+-- Since 3.3.8
+setMaxTotalHeaderLength :: Int -> Settings -> Settings
+setMaxTotalHeaderLength maxTotalHeaderLength settings = settings
+  { settingsMaxTotalHeaderLength = maxTotalHeaderLength }
+
+
+-- | Setting the header value of Alternative Services (AltSvc:).
+--
+-- Since 3.3.11
+setAltSvc :: ByteString -> Settings -> Settings
+setAltSvc altsvc settings = settings { settingsAltSvc = Just altsvc }
+
 -- | Explicitly pause the slowloris timeout.
 --
 -- This is useful for cases where you partially consume a request body. For
@@ -457,3 +485,39 @@ pauseTimeout = fromMaybe (return ()) . Vault.lookup pauseTimeoutKey . vault
 -- Since 3.1.10
 getFileInfo :: Request -> FilePath -> IO FileInfo
 getFileInfo = fromMaybe (\_ -> throwIO (userError "getFileInfo")) . Vault.lookup getFileInfoKey . vault
+
+-- | A timeout to limit the time (in milliseconds) waiting for
+--   FIN for HTTP/1.x. 0 means uses immediate close.
+--   Default: 0.
+--
+-- Since 3.3.5
+setGracefulCloseTimeout1 :: Int -> Settings -> Settings
+setGracefulCloseTimeout1 x y = y { settingsGracefulCloseTimeout1 = x }
+
+-- | A timeout to limit the time (in milliseconds) waiting for
+--   FIN for HTTP/1.x. 0 means uses immediate close.
+--
+-- Since 3.3.5
+getGracefulCloseTimeout1 :: Settings -> Int
+getGracefulCloseTimeout1 = settingsGracefulCloseTimeout1
+
+-- | A timeout to limit the time (in milliseconds) waiting for
+--   FIN for HTTP/2. 0 means uses immediate close.
+--   Default: 2000.
+--
+-- Since 3.3.5
+setGracefulCloseTimeout2 :: Int -> Settings -> Settings
+setGracefulCloseTimeout2 x y = y { settingsGracefulCloseTimeout2 = x }
+
+-- | A timeout to limit the time (in milliseconds) waiting for
+--   FIN for HTTP/2. 0 means uses immediate close.
+--
+-- Since 3.3.5
+getGracefulCloseTimeout2 :: Settings -> Int
+getGracefulCloseTimeout2 = settingsGracefulCloseTimeout2
+
+-- | Getting information of client certificate.
+--
+-- Since 3.3.5
+clientCertificate :: Request -> Maybe CertificateChain
+clientCertificate = join . Vault.lookup getClientCertificateKey . vault
