@@ -70,6 +70,7 @@ module Network.Wai.Handler.Warp.Internal (
     -- * Misc
   , http2server
   , withII
+  , pReadMaker
   ) where
 
 import System.TimeManager
@@ -79,6 +80,7 @@ import Network.Wai.Handler.Warp.Date
 import Network.Wai.Handler.Warp.FdCache
 import Network.Wai.Handler.Warp.FileInfoCache
 import Network.Wai.Handler.Warp.HTTP2
+import Network.Wai.Handler.Warp.HTTP2.File
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.Recv
 import Network.Wai.Handler.Warp.Request
