@@ -11,6 +11,7 @@ module Network.Wai.Handler.Warp.Response (
   , hasBody
   , replaceHeader
   , addServer -- testing
+  , addAltSvc
   ) where
 
 import Data.ByteString.Builder.HTTP.Chunked (chunkedTransferEncoding, chunkedTransferTerminator)
@@ -108,7 +109,7 @@ sendResponse :: Settings
              -> Response -- ^ HTTP response including status code and response header.
              -> IO Bool -- ^ Returing True if the connection is persistent.
 sendResponse settings conn ii th req reqidxhdr src response = do
-    hs <- addServerAndDate hs0
+    hs <- addAltSvc settings <$> addServerAndDate hs0
     if hasBody s then do
         -- The response to HEAD does not have body.
         -- But to handle the conditional requests defined RFC 7232 and
@@ -116,14 +117,14 @@ sendResponse settings conn ii th req reqidxhdr src response = do
         -- and status, the response to HEAD is processed here.
         --
         -- See definition of rsp below for proper body stripping.
-        (ms, mlen) <- sendRsp conn ii ver s hs rsp
+        (ms, mlen) <- sendRsp conn ii th ver s hs rspidxhdr rsp
         case ms of
             Nothing         -> return ()
             Just realStatus -> logger req realStatus mlen
         T.tickle th
         return ret
       else do
-        _ <- sendRsp conn ii ver s hs RspNoBody
+        _ <- sendRsp conn ii th ver s hs rspidxhdr RspNoBody
         logger req s Nothing
         T.tickle th
         return isPersist
@@ -147,8 +148,8 @@ sendResponse settings conn ii th req reqidxhdr src response = do
           | otherwise               -> RspBuilder b needsChunked
         ResponseStream _ _ fb
           | isHead                  -> RspNoBody
-          | otherwise               -> RspStream fb needsChunked th
-        ResponseRaw raw _           -> RspRaw raw src (T.tickle th)
+          | otherwise               -> RspStream fb needsChunked
+        ResponseRaw raw _           -> RspRaw raw src
     -- Make sure we don't hang on to 'response' (avoid space leak)
     !ret = case response of
         ResponseFile    {} -> isPersist
@@ -186,22 +187,24 @@ sanitizeHeaderValue v = case C8.lines $ S.filter (/= _cr) v of
 data Rsp = RspNoBody
          | RspFile FilePath (Maybe FilePart) IndexedHeader Bool (IO ())
          | RspBuilder Builder Bool
-         | RspStream StreamingBody Bool T.Handle
-         | RspRaw (IO ByteString -> (ByteString -> IO ()) -> IO ()) (IO ByteString) (IO ())
+         | RspStream StreamingBody Bool
+         | RspRaw (IO ByteString -> (ByteString -> IO ()) -> IO ()) (IO ByteString)
 
 ----------------------------------------------------------------
 
 sendRsp :: Connection
         -> InternalInfo
+        -> T.Handle
         -> H.HttpVersion
         -> H.Status
         -> H.ResponseHeaders
+        -> IndexedHeader -- Response
         -> Rsp
         -> IO (Maybe H.Status, Maybe Integer)
 
 ----------------------------------------------------------------
 
-sendRsp conn _ ver s hs RspNoBody = do
+sendRsp conn _ _ ver s hs _ RspNoBody = do
     -- Not adding Content-Length.
     -- User agents treats it as Content-Length: 0.
     composeHeader ver s hs >>= connSendAll conn
@@ -209,7 +212,7 @@ sendRsp conn _ ver s hs RspNoBody = do
 
 ----------------------------------------------------------------
 
-sendRsp conn _ ver s hs (RspBuilder body needsChunked) = do
+sendRsp conn _ th ver s hs _ (RspBuilder body needsChunked) = do
     header <- composeHeaderBuilder ver s hs needsChunked
     let hdrBdy
          | needsChunked = header <> chunkedTransferEncoding body
@@ -217,12 +220,12 @@ sendRsp conn _ ver s hs (RspBuilder body needsChunked) = do
          | otherwise    = header <> body
         buffer = connWriteBuffer conn
         size = connBufferSize conn
-    toBufIOWith buffer size (connSendAll conn) hdrBdy
+    toBufIOWith buffer size (\bs -> connSendAll conn bs >> T.tickle th) hdrBdy
     return (Just s, Nothing) -- fixme: can we tell the actual sent bytes?
 
 ----------------------------------------------------------------
 
-sendRsp conn _ ver s hs (RspStream streamingBody needsChunked th) = do
+sendRsp conn _ th ver s hs _ (RspStream streamingBody needsChunked) = do
     header <- composeHeaderBuilder ver s hs needsChunked
     (recv, finish) <- newByteStringBuilderRecv $ reuseBufferStrategy
                     $ toBuilderBuffer (connWriteBuffer conn) (connBufferSize conn)
@@ -246,22 +249,22 @@ sendRsp conn _ ver s hs (RspStream streamingBody needsChunked th) = do
 
 ----------------------------------------------------------------
 
-sendRsp conn _ _ _ _ (RspRaw withApp src tickle) = do
+sendRsp conn _ th _ _ _ _ (RspRaw withApp src) = do
     withApp recv send
     return (Nothing, Nothing)
   where
     recv = do
         bs <- src
-        unless (S.null bs) tickle
+        unless (S.null bs) $ T.tickle th
         return bs
-    send bs = connSendAll conn bs >> tickle
+    send bs = connSendAll conn bs >> T.tickle th
 
 ----------------------------------------------------------------
 
 -- Sophisticated WAI applications.
 -- We respect s0. s0 MUST be a proper value.
-sendRsp conn ii ver s0 hs0 (RspFile path (Just part) _ isHead hook) =
-    sendRspFile2XX conn ii ver s0 hs path beg len isHead hook
+sendRsp conn ii th ver s0 hs0 rspidxhdr (RspFile path (Just part) _ isHead hook) =
+    sendRspFile2XX conn ii th ver s0 hs rspidxhdr path beg len isHead hook
   where
     beg = filePartOffset part
     len = filePartByteCount part
@@ -271,33 +274,35 @@ sendRsp conn ii ver s0 hs0 (RspFile path (Just part) _ isHead hook) =
 
 -- Simple WAI applications.
 -- Status is ignored
-sendRsp conn ii ver _ hs0 (RspFile path Nothing idxhdr isHead hook) = do
+sendRsp conn ii th ver _ hs0 rspidxhdr (RspFile path Nothing reqidxhdr isHead hook) = do
     efinfo <- E.try $ getFileInfo ii path
     case efinfo of
         Left (_ex :: E.IOException) ->
 #ifdef WARP_DEBUG
           print _ex >>
 #endif
-          sendRspFile404 conn ii ver hs0
-        Right finfo -> case conditionalRequest finfo hs0 idxhdr of
-          WithoutBody s         -> sendRsp conn ii ver s hs0 RspNoBody
-          WithBody s hs beg len -> sendRspFile2XX conn ii ver s hs path beg len isHead hook
+          sendRspFile404 conn ii th ver hs0 rspidxhdr
+        Right finfo -> case conditionalRequest finfo hs0 rspidxhdr reqidxhdr of
+          WithoutBody s         -> sendRsp conn ii th ver s hs0 rspidxhdr RspNoBody
+          WithBody s hs beg len -> sendRspFile2XX conn ii th ver s hs rspidxhdr path beg len isHead hook
 
 ----------------------------------------------------------------
 
 sendRspFile2XX :: Connection
                -> InternalInfo
+               -> T.Handle
                -> H.HttpVersion
                -> H.Status
                -> H.ResponseHeaders
+               -> IndexedHeader
                -> FilePath
                -> Integer
                -> Integer
                -> Bool
                -> IO ()
                -> IO (Maybe H.Status, Maybe Integer)
-sendRspFile2XX conn ii ver s hs path beg len isHead hook
-  | isHead = sendRsp conn ii ver s hs RspNoBody
+sendRspFile2XX conn ii th ver s hs rspidxhdr path beg len isHead hook
+  | isHead = sendRsp conn ii th ver s hs rspidxhdr RspNoBody
   | otherwise = do
       lheader <- composeHeader ver s hs
       (mfd, fresher) <- getFd ii path
@@ -308,10 +313,12 @@ sendRspFile2XX conn ii ver s hs path beg len isHead hook
 
 sendRspFile404 :: Connection
                -> InternalInfo
+               -> T.Handle
                -> H.HttpVersion
                -> H.ResponseHeaders
+               -> IndexedHeader
                -> IO (Maybe H.Status, Maybe Integer)
-sendRspFile404 conn ii ver hs0 = sendRsp conn ii ver s hs (RspBuilder body True)
+sendRspFile404 conn ii th ver hs0 rspidxhdr = sendRsp conn ii th ver s hs rspidxhdr (RspBuilder body True)
   where
     s = H.notFound404
     hs =  replaceHeader H.hContentType "text/plain; charset=utf-8" hs0
@@ -406,6 +413,11 @@ addServer serverName rspidxhdr hdrs = case rspidxhdr ! fromEnum ResServer of
     Nothing -> (H.hServer, serverName) : hdrs
     _       -> hdrs
 
+addAltSvc :: Settings -> H.ResponseHeaders -> H.ResponseHeaders
+addAltSvc settings hs = case settingsAltSvc settings of
+                Nothing -> hs
+                Just  v -> ("Alt-Svc", v) : hs
+
 ----------------------------------------------------------------
 
 -- |
