@@ -3,8 +3,10 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PackageImports #-}
 {-# LANGUAGE PatternGuards #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TupleSections #-}
+{-# LANGUAGE ViewPatterns #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
 module Network.Wai.Handler.Warp.Run where
@@ -15,24 +17,30 @@ import qualified Control.Concurrent as Conc (yield)
 import Control.Exception as E
 import qualified Data.ByteString as S
 import Data.Char (chr)
-import Data.IORef (IORef, newIORef, readIORef, writeIORef, atomicModifyIORef')
+import Data.IORef (IORef, newIORef, readIORef, writeIORef)
 import Data.Streaming.Network (bindPortTCP)
 import Foreign.C.Error (Errno(..), eCONNABORTED)
-import GHC.IO.Exception (IOException(..))
+import GHC.IO.Exception (IOException(..), IOErrorType(..))
+import qualified Network.HTTP2 as H2
 import Network.Socket (Socket, close, accept, withSocketsDo, SockAddr(SockAddrInet, SockAddrInet6), setSocketOption, SocketOption(..))
+#if MIN_VERSION_network(3,1,1)
+import Network.Socket (gracefulClose)
+#endif
 import qualified Network.Socket.ByteString as Sock
 import Network.Wai
 import Network.Wai.Internal (ResponseReceived (ResponseReceived))
 import System.Environment (lookupEnv)
-import System.Timeout (timeout)
+import System.IO.Error (ioeGetErrorType)
 import qualified System.TimeManager as T
+import System.Timeout (timeout)
 
 import Network.Wai.Handler.Warp.Buffer
 import Network.Wai.Handler.Warp.Counter
 import qualified Network.Wai.Handler.Warp.Date as D
 import qualified Network.Wai.Handler.Warp.FdCache as F
 import qualified Network.Wai.Handler.Warp.FileInfoCache as I
-import Network.Wai.Handler.Warp.HTTP2 (http2, isHTTP2)
+import Network.Wai.Handler.Warp.HTTP2 (http2)
+import Network.Wai.Handler.Warp.HTTP2.Types (isHTTP2)
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.Imports hiding (readInt)
 import Network.Wai.Handler.Warp.ReadInt
@@ -51,22 +59,46 @@ import Network.Socket (fdSocket)
 #endif
 
 -- | Creating 'Connection' for plain HTTP based on a given socket.
-socketConnection :: Socket -> IO Connection
-socketConnection s = do
+socketConnection :: Settings -> Socket -> IO Connection
+#if MIN_VERSION_network(3,1,1)
+socketConnection set s = do
+#else
+socketConnection _ s = do
+#endif
     bufferPool <- newBufferPool
     writeBuf <- allocateBuffer bufferSize
-    let sendall = Sock.sendAll s
+    let sendall = sendAll' s
+    isH2 <- newIORef False -- HTTP/1.x
     return Connection {
         connSendMany = Sock.sendMany s
       , connSendAll = sendall
       , connSendFile = sendFile s writeBuf bufferSize sendall
+#if MIN_VERSION_network(3,1,1)
+      , connClose = do
+            h2 <- readIORef isH2
+            let tm = if h2 then settingsGracefulCloseTimeout2 set
+                           else settingsGracefulCloseTimeout1 set
+            if tm == 0 then
+                close s
+              else
+                gracefulClose s tm `E.catch` \(E.SomeException _) -> return ()
+#else
       , connClose = close s
+#endif
       , connFree = freeBuffer writeBuf
       , connRecv = receive s bufferPool
       , connRecvBuf = receiveBuf s
       , connWriteBuffer = writeBuf
       , connBufferSize = bufferSize
+      , connHTTP2 = isH2
       }
+  where
+    sendAll' sock bs = E.handleJust
+      (\ e -> if ioeGetErrorType e == ResourceVanished
+        then Just ConnectionClosedByPeer
+        else Nothing)
+      throwIO
+      $ Sock.sendAll sock bs
 
 -- | Run an 'Application' on the given port.
 -- This calls 'runSettings' with 'defaultSettings'.
@@ -127,7 +159,7 @@ runSettingsSocket set socket app = do
         setSocketCloseOnExec s
         -- NoDelay causes an error for AF_UNIX.
         setSocketOption s NoDelay 1 `E.catch` \(E.SomeException _) -> return ()
-        conn <- socketConnection s
+        conn <- socketConnection set s
         return (conn, sa)
 
     closeListenSocket = close socket
@@ -168,16 +200,20 @@ runSettingsConnectionMakerSecure :: Settings -> IO (IO (Connection, Transport),
 runSettingsConnectionMakerSecure set getConnMaker app = do
     settingsBeforeMainLoop set
     counter <- newCounter
-    withII $ acceptConnection set getConnMaker app counter
-  where
-    withII action =
-        withTimeoutManager $ \tm ->
-        D.withDateCache $ \dc ->
-        F.withFdCache fdCacheDurationInSeconds $ \fdc ->
-        I.withFileInfoCache fdFileInfoDurationInSeconds $ \fic -> do
-            let ii = InternalInfo tm dc fdc fic
-            action ii
+    withII set $ acceptConnection set getConnMaker app counter
 
+-- | Running an action with internal info.
+--
+-- Since 3.3.11
+withII :: Settings -> (InternalInfo -> IO a) -> IO a
+withII set action =
+    withTimeoutManager $ \tm ->
+    D.withDateCache $ \dc ->
+    F.withFdCache fdCacheDurationInSeconds $ \fdc ->
+    I.withFileInfoCache fdFileInfoDurationInSeconds $ \fic -> do
+        let ii = InternalInfo tm dc fdc fic
+        action ii
+  where
     !fdCacheDurationInSeconds = settingsFdCacheDuration set * 1000000
     !fdFileInfoDurationInSeconds = settingsFileInfoCacheDuration set * 1000000
     !timeoutInSeconds = settingsTimeout set * 1000000
@@ -262,10 +298,7 @@ fork :: Settings
 fork set mkConn addr app counter ii = settingsFork set $ \unmask ->
     -- Call the user-supplied on exception code if any
     -- exceptions are thrown.
-    handle (settingsOnException set Nothing) .
-    -- Allocate a new IORef indicating whether the connection has been
-    -- closed, to avoid double-freeing a connection
-    withClosedRef $ \ref ->
+    handle (settingsOnException set Nothing) $
         -- Run the connection maker to get a new connection, and ensure
         -- that the connection is closed. If the mkConn call throws an
         -- exception, we will leak the connection. If the mkConn call is
@@ -276,21 +309,13 @@ fork set mkConn addr app counter ii = settingsFork set $ \unmask ->
         -- We grab the connection before registering timeouts since the
         -- timeouts will be useless during connection creation, due to the
         -- fact that async exceptions are still masked.
-        bracket mkConn (cleanUp ref) (serve unmask ref)
+        bracket mkConn cleanUp (serve unmask)
   where
-    withClosedRef inner = newIORef False >>= inner
-
-    closeConn ref conn = do
-        isClosed <- atomicModifyIORef' ref $ \x -> (True, x)
-        unless isClosed $ connClose conn
-
-    cleanUp ref (conn, _) = closeConn ref conn `finally` connFree conn
+    cleanUp (conn, _) = connClose conn `finally` connFree conn
 
     -- We need to register a timeout handler for this thread, and
-    -- cancel that handler as soon as we exit. We additionally close
-    -- the connection immediately in case the child thread catches the
-    -- async exception or performs some long-running cleanup action.
-    serve unmask ref (conn, transport) = bracket register cancel $ \th -> do
+    -- cancel that handler as soon as we exit.
+    serve unmask (conn, transport) = bracket register cancel $ \th -> do
         -- We now have fully registered a connection close handler in
         -- the case of all exceptions, so it is safe to once again
         -- allow async exceptions.
@@ -302,8 +327,7 @@ fork set mkConn addr app counter ii = settingsFork set $ \unmask ->
            -- above ensures the connection is closed.
            when goingon $ serveConnection conn ii th addr transport set app
       where
-        register = T.registerKillThread (timeoutManager ii)
-                                        (closeConn ref conn)
+        register = T.registerKillThread (timeoutManager ii) (connClose conn)
         cancel   = T.cancel
 
     onOpen adr    = increase counter >> settingsOnOpen  set adr
@@ -330,22 +354,35 @@ serveConnection conn ii th origAddr transport settings app = do
     istatus <- newIORef False
     if settingsHTTP2Enabled settings && h2 then do
         rawRecvN <- makeReceiveN bs (connRecv conn) (connRecvBuf conn)
+        -- This thread becomes the sender in http2 library.
+        -- In the case of event source, one request comes and one
+        -- worker gets busy. But it is likely that the receiver does
+        -- not receive any data at all while the sender is sending
+        -- output data from the worker. It's not good enough to tickle
+        -- the time handler in the receiver only. So, we should tickle
+        -- the time handler in both the receiver and the sender.
         let recvN = wrappedRecvN th istatus (settingsSlowlorisSize settings) rawRecvN
+            sendBS x = connSendAll conn x >> T.tickle th
         -- fixme: origAddr
-        http2 conn ii origAddr transport settings recvN app
+        checkTLS
+        setConnHTTP2 conn True
+        http2 settings ii conn transport origAddr recvN sendBS app
       else do
         src <- mkSource (wrappedRecv conn th istatus (settingsSlowlorisSize settings))
         writeIORef istatus True
         leftoverSource src bs
         addr <- getProxyProtocolAddr src
         http1 True addr istatus src `E.catch` \e ->
-          case fromException e of
-            -- See comment below referencing
-            -- https://github.com/yesodweb/wai/issues/618
-            Just NoKeepAliveRequest -> return ()
-            Nothing -> do
-              _ <- sendErrorResponse (dummyreq addr) istatus e
-              throwIO e
+          case () of
+            ()
+             -- See comment below referencing
+             -- https://github.com/yesodweb/wai/issues/618
+             | Just NoKeepAliveRequest <- fromException e -> return ()
+             -- No valid request
+             | Just (BadFirstLine _)   <- fromException e -> return ()
+             | otherwise -> do
+               _ <- sendErrorResponse (dummyreq addr) istatus e
+               throwIO e
 
   where
     getProxyProtocolAddr src =
@@ -404,8 +441,7 @@ serveConnection conn ii th origAddr transport settings app = do
     errorResponse e = settingsOnExceptionResponse settings e
 
     http1 firstRequest addr istatus src = do
-        (req', mremainingRef, idxhdr, nextBodyFlush) <- recvRequest firstRequest settings conn ii th addr src
-        let req = req' { isSecure = isTransportSecure transport }
+        (req, mremainingRef, idxhdr, nextBodyFlush) <- recvRequest firstRequest settings conn ii th addr src transport
         keepAlive <- processRequest istatus src req mremainingRef idxhdr nextBodyFlush
             `E.catch` \e -> do
                 settingsOnException settings (Just req) e
@@ -490,6 +526,19 @@ serveConnection conn ii th origAddr transport settings app = do
           else
             return False
 
+    checkTLS = case transport of
+        TCP -> return () -- direct
+        tls -> unless (tls12orLater tls) $ goaway conn H2.InadequateSecurity "Weak TLS"
+    tls12orLater tls = tlsMajorVersion tls == 3 && tlsMinorVersion tls >= 3
+
+-- connClose must not be called here since Run:fork calls it
+goaway :: Connection -> H2.ErrorCodeId -> ByteString -> IO ()
+goaway Connection{..} etype debugmsg = connSendAll bytestream
+  where
+    einfo = H2.encodeInfo id 0
+    frame = H2.GoAwayFrame 0 etype debugmsg
+    bytestream = H2.encodeFrame einfo frame
+
 flushEntireBody :: IO ByteString -> IO ()
 flushEntireBody src =
     loop
