@@ -10,7 +10,7 @@ import Network.HTTP.Types
 import Network.Wai hiding (responseHeaders)
 import Network.Wai.Handler.Warp
 import Network.Wai.Handler.Warp.Response
-import RunSpec (withApp, msClose, msWrite, msRead, connectTo)
+import RunSpec (withApp, msWrite, msRead, withMySocket)
 import Test.Hspec
 
 main :: IO ()
@@ -20,15 +20,13 @@ testRange :: S.ByteString -- ^ range value
           -> String -- ^ expected output
           -> Maybe String -- ^ expected content-range value
           -> Spec
-testRange range out crange = it title $ withApp defaultSettings app $ \port -> do
-    handle <- connectTo port
-    msWrite handle "GET / HTTP/1.0\r\n"
-    msWrite handle "Range: bytes="
-    msWrite handle range
-    msWrite handle "\r\n\r\n"
+testRange range out crange = it title $ withApp defaultSettings app $ withMySocket $ \ms -> do
+    msWrite ms "GET / HTTP/1.0\r\n"
+    msWrite ms "Range: bytes="
+    msWrite ms range
+    msWrite ms "\r\n\r\n"
     threadDelay 10000
-    bss <- fmap (lines . filter (/= '\r') . S8.unpack) $ msRead handle 1024
-    msClose handle
+    bss <- fmap (lines . filter (/= '\r') . S8.unpack) $ msRead ms 1024
     last bss `shouldBe` out
     let hs = mapMaybe toHeader bss
     lookup "Content-Range" hs `shouldBe` fmap ("bytes " ++) crange
@@ -46,12 +44,10 @@ testPartial :: Integer -- ^ file size
             -> Integer -- ^ byte count
             -> String -- ^ expected output
             -> Spec
-testPartial size offset count out = it title $ withApp defaultSettings app $ \port -> do
-    handle <- connectTo port
-    msWrite handle "GET / HTTP/1.0\r\n\r\n"
+testPartial size offset count out = it title $ withApp defaultSettings app $ withMySocket $ \ms -> do
+    msWrite ms "GET / HTTP/1.0\r\n\r\n"
     threadDelay 10000
-    bss <- fmap (lines . filter (/= '\r') . S8.unpack) $ msRead handle 1024
-    msClose handle
+    bss <- fmap (lines . filter (/= '\r') . S8.unpack) $ msRead ms 1024
     out `shouldBe` last bss
     let hs = mapMaybe toHeader bss
     lookup "Content-Length" hs `shouldBe` Just (show $ length $ last bss)
@@ -87,7 +83,7 @@ spec = do
         it "discards empty lines" $ do
             sanitizeHeaderValue "foo\r\n\r\nbar" `shouldBe` "foo\r\n bar"
 
-        context "when sanitizing single occurences of \n" $ do
+        context "when sanitizing single occurrences of \n" $ do
             it "replaces \n with \r\n" $ do
                 sanitizeHeaderValue "foo\n bar" `shouldBe` "foo\r\n bar"
 
